//  Program:  alphabetize.cpp
//  Author:   Chuck Stewart
//  Purpose:  Demonstrate using a vector of strings and sorting.

#include <algorithm>
#include <fstream>
#include <iostream>
#include <string>
#include <vector>

//  This is used for sorting in reverse alphabetical order, if we
//  wish.  The code for doing so is commented out below.

bool greater_on_strings( std::string const& left, std::string const& right )
{
  return right < left;
}

int
main( int argc, char* argv[] )
{
  if ( argc != 3 )
    {
      std::cerr << "Usage: " << argv[0] << " names-in names-out\n";
      return 1;
    }

  //  Attach an input stream to the file specified in the first
  //  command-line argument.
  std::ifstream names_in_str( argv[1] );
  if ( !names_in_str )
    {
      std::cerr << "Can not open the names file " << argv[1] << "\n";
      return 1;
    }

  //  Create the output file specified in the second command-line
  //  argument, attaching an output stream to it.
  std::ofstream names_out_str( argv[2] );
  if ( !names_out_str )
    {
      std::cerr << "Can not open the output names file " << argv[2] << "\n";
      return 1;
    }

  std::vector<std::string> names;
  std::string one_name;

  //  Read the strings one at a time and add them to the back of the
  //  vector.  The reading loop ends when the end of file has been reached. 
  while ( names_in_str >> one_name )
    {
      names.push_back( one_name );
    }

  //  Sort the vector of strings in the same manner that we sorted the
  //  vector of doubles.  The sort function uses (automatically) the <
  //  operator which is defined on strings.  This operator compares
  //  strings "lexicographically".

  std::sort( names.begin(), names.end() );

  //  Here is commented-out code for arranging strings in reverse
  //  lexicographic order.  It has a third argument to the sort
  //  function --- the name of the function to use for comparisons.
  //  This function must return a bool and it must have two arguments
  //  --- const references to the strings that are to be compared.  It
  //  returns true if the first argument should come before the second
  //  argument in the sorted order.

  // std::sort( names.begin(), names.end(), greater_on_strings );


  //  Now just output.
  names_out_str << "\n" << "Here are the names in alphabetical order." << std::endl;
  for ( unsigned int i=0; i<names.size(); ++i )
    {
      names_out_str << names[i] << '\n';
    }

  return 0;
}
