
//  Program:  average_and_deviation.cpp
//  Author:   Chuck Stewart
//  Purpose:  Compute the average and standard deviation of an input
//    set of grades.   This introduces the use of a vector to store
//    the grades upon input.

#include <cmath>
#include <fstream>
#include <iomanip>
#include <iostream>
#include <vector>

//  You will notice that the main function now has parameters.  These
//  provide the program with what was typed on the command line as a
//  sequence of C-style strings.  argc is the number of arguments,
//  while argv is an array of C-style character strings (char*).  The
//  first value, argv[0], is always the name of the program.

int main( int argc, char* argv[] )
{
  //  Make sure there are two arguments.
  if ( argc != 2 )
    {
      std::cerr << "Usage: " << argv[0] << " grades-file\n";
      return 1;
    }

  //  Now we attach an input stream to the file named in argument 1.
  //  The constructor for the ifstream object called grades_str
  //  attempts to open the file.  If it is unsuccessful in doing so,
  //  then when we apply the ! operator to the grades_str object, the
  //  operator will return false.
  std::ifstream grades_str( argv[1] );
  if ( !grades_str )
    {
      std::cerr << "Can not open the grades file " << argv[1] << "\n";
      return 1;
    }

  std::vector<int> scores;  // Vector to hold the input scores; initially empty.
  int x;                    // Input variable

  //  Read the scores, appending each to the end of the vector.  Note
  //  how grades_str is now treated the same way as std::cin.
  while ( grades_str >> x )
    {
      scores.push_back(x);
    }

  //  Quit with an error message if there are too few scores.
  if ( scores.size() == 0 )
    {
      std::cout << "No scores entered.  Please try again!" << std::endl;
      return 1;  
    }

  //  Compute and output the average value.
  int sum=0;           // Accumulation of the values
  for ( unsigned int i = 0; i < scores.size(); ++ i )
    {
      sum += scores[i];
    }
  double average = double(sum) / scores.size();

  //  In the following output, by sending std::setprecision(3) to the
  //  output stream, we are telling it to output at most 3 significant
  //  digits.  This will remain in effect for all subsequent outputs.
  //  Output stream have a number of ways to manipulate the form of
  //  the output.
  std::cout << "The average of " << scores.size()
	    << " grades is " << std::setprecision(3)
	    << average << std::endl;

  //  Compute and output the standard deviation.  Note that this will
  //  crash if only one score is read.  We should be more careful.
  double sum_sq_diff = 0.0;
  for ( unsigned int i=0; i<scores.size(); ++i )
    {
      double diff = scores[i] - average;
      sum_sq_diff += diff*diff;
    }
  double std_dev = sqrt( sum_sq_diff / (scores.size()-1) );
  std::cout << "The standard deviation is " << std_dev << std::endl;

  return 0;  //  Everything ok
}
