
#include <iostream>
#include <vector>
using namespace std;


//  Here is the recursive function.  The "invariant" is that if x is
//  in the vector then it must be located within the subscript range
//  low to high.  Therefore, when low and high are equal, their common
//  value is the only possible place for x.  Otherwise, the middle
//  value is checked, and the search continues recursively in either
//  the lower or upper half of the vector.

bool binsearch( const vector<double>& v, int low, int high, double x )
{
  if ( high == low ) return x == v[low];

  int mid = (low+high) / 2;
  if ( x <= v[mid] )
    return binsearch( v, low, mid, x );
  else
    return binsearch( v, mid+1, high, x );
}


//  The driver function.  It establishes the search range for the
//  value of x based on the minimum and maximum subscripts in the
//  vector.

bool binsearch( const vector<double>& v, double x )
{
  return binsearch( v, 0, v.size()-1, x );
}
  

int main()
{
  vector<double> a;
  a.push_back( 1.0 );
  a.push_back( 1.25 );
  a.push_back( 2.8 );
  a.push_back( 4.0 );
  a.push_back( 5.4 );
  a.push_back( 7.1 );

  cout << "Looking for -1: " << binsearch( a, -1 ) << endl
       << "Looking for 1: " << binsearch( a, 1.0 ) << endl
       << "Looking for 1.1: " << binsearch( a, 1.1 ) << endl
       << "Looking for 1.25: " << binsearch( a, 1.25 ) << endl
       << "Looking for 7.1: " << binsearch( a, 7.1 ) << endl
       << "Looking for 7.4: " << binsearch( a, 7.4 ) << endl;
  return 0;
}
    
